/*****************************************************************************
 * modules_plugin.h : Plugin management functions used by the core application.
 *****************************************************************************
 * Copyright (C) 2001 VideoLAN
 * Automatically generated from ./src/misc/modules_plugin.h.in by bootstrap
 *
 * Authors: Samuel Hocevar <sam@zoy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

/*****************************************************************************
 * Inline functions for handling dynamic modules
 *****************************************************************************/

/*****************************************************************************
 * module_load: load a dynamic library
 *****************************************************************************
 * This function loads a dynamically linked library using a system dependant
 * method, and returns a non-zero value on error, zero otherwise.
 *****************************************************************************/
static int module_load( const MYCHAR * psz_filename, module_handle_t * handle )
{
#ifdef SYS_BEOS
    *handle = load_add_on( psz_filename );
    return( *handle < 0 );

#elif defined(WIN32) || defined(UNDER_CE)
    *handle = LoadLibrary( psz_filename );
    return( *handle == NULL );

#elif defined(RTLD_NOW)
    /* static is OK, we are called atomically */
    static vlc_bool_t b_kde = VLC_FALSE;

#   if defined(SYS_LINUX)
    /* XXX HACK #1 - we should NOT open modules with RTLD_GLOBAL, or we
     * are going to get namespace collisions when two modules have common
     * public symbols, but ALSA is being a pest here. */
    if( strstr( psz_filename, "alsa_plugin" ) )
    {
        *handle = dlopen( psz_filename, RTLD_NOW | RTLD_GLOBAL );
        return( *handle == NULL );
    }
#   endif
    /* XXX HACK #2 - the ugly KDE workaround. It seems that libkdewhatever
     * causes dlopen() to segfault if libstdc++ is not loaded in the caller,
     * so we just load libstdc++. Bwahahaha! ph34r! -- Sam. */
    /* Update: FYI, this is Debian bug #180505, and seems to be fixed. */
    if( !b_kde && !strstr( psz_filename, "kde" ) )
    {
        dlopen( "libstdc++.so.6", RTLD_NOW )
         || dlopen( "libstdc++.so.5", RTLD_NOW )
         || dlopen( "libstdc++.so.4", RTLD_NOW )
         || dlopen( "libstdc++.so.3", RTLD_NOW );
        b_kde = VLC_TRUE;
    }

    *handle = dlopen( psz_filename, RTLD_NOW );
    return( *handle == NULL );

#else
    *handle = dlopen( psz_filename, DL_LAZY );
    return( *handle == NULL );

#endif
}

/*****************************************************************************
 * module_unload: unload a dynamic library
 *****************************************************************************
 * This function unloads a previously opened dynamically linked library
 * using a system dependant method. No return value is taken in consideration,
 * since some libraries sometimes refuse to close properly.
 *****************************************************************************/
static void module_unload( module_handle_t handle )
{
#ifdef SYS_BEOS
    unload_add_on( handle );

#elif defined(WIN32) || defined(UNDER_CE)
    FreeLibrary( handle );

#else
    dlclose( handle );

#endif
    return;
}

/*****************************************************************************
 * module_getsymbol: get a symbol from a dynamic library
 *****************************************************************************
 * This function queries a loaded library for a symbol specified in a
 * string, and returns a pointer to it. We don't check for dlerror() or
 * similar functions, since we want a non-NULL symbol anyway.
 *****************************************************************************/
static void * _module_getsymbol( module_handle_t handle,
                                 const char * psz_function )
{
#ifdef SYS_BEOS
    void * p_symbol;
    if( B_OK == get_image_symbol( handle, psz_function,
                                  B_SYMBOL_TYPE_TEXT, &p_symbol ) )
    {
        return p_symbol;
    }
    else
    {
        return NULL;
    }

#elif defined( UNDER_CE )
    wchar_t psz_real[256];
    MultiByteToWideChar( CP_ACP, 0, psz_function, -1, psz_real, 256 );

    return (void *)GetProcAddress( handle, psz_real );

#elif defined( WIN32 )
    return (void *)GetProcAddress( handle, (MYCHAR*)psz_function );

#else
    return dlsym( handle, psz_function );

#endif
}

static void * module_getsymbol( module_handle_t handle,
                                const char * psz_function )
{
    void * p_symbol = _module_getsymbol( handle, psz_function );

    /* MacOS X dl library expects symbols to begin with "_". So do
     * some other operating systems. That's really lame, but hey, what
     * can we do ? */
    if( p_symbol == NULL )
    {
        char *psz_call = malloc( strlen( psz_function ) + 2 );

        strcpy( psz_call + 1, psz_function );
        psz_call[ 0 ] = '_';
        p_symbol = _module_getsymbol( handle, psz_call );
        free( psz_call );
    }

    return p_symbol;
}

/*****************************************************************************
 * module_error: wrapper for dlerror()
 *****************************************************************************
 * This function returns the error message of the last module operation. It
 * returns the string "failed" on systems which do not have a dlerror() like
 * function. psz_buffer can be used to store temporary data, it is guaranteed
 * to be kept intact until the return value of module_error has been used.
 *****************************************************************************/
static const char * module_error( char *psz_buffer )
{
#if defined(SYS_BEOS)
    return "failed";

#elif defined(UNDER_CE)
    wchar_t psz_tmp[256];

    int i, i_error = GetLastError();

    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL, i_error, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) psz_tmp, 256, NULL );

    /* Go to the end of the string */
    for( i = 0;
         psz_tmp[i] && psz_tmp[i] != L'\r' && psz_tmp[i] != L'\n';
         i++ ) {};

    if( psz_tmp[i] )
    {
        swprintf( psz_tmp + i, L" (error %i)", i_error );
        psz_tmp[ 255 ] = L'\0';
    }

    WideCharToMultiByte( CP_ACP, WC_DEFAULTCHAR, psz_tmp, -1,
                         psz_buffer, 256, NULL, NULL );

    return psz_buffer;

#elif defined(WIN32)
    int i, i_error = GetLastError();

    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL, i_error, MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                   (LPTSTR) psz_buffer, 256, NULL );

    /* Go to the end of the string */
    for( i = 0;
         psz_buffer[i] && psz_buffer[i] != '\r' && psz_buffer[i] != '\n';
         i++ ) {};

    if( psz_buffer[i] )
    {
        snprintf( psz_buffer + i, 256 - i, " (error %i)", i_error );
        psz_buffer[ 255 ] = '\0';
    }

    return psz_buffer;

#else
    return dlerror();

#endif
}

/*****************************************************************************
 * STORE_SYMBOLS: store known symbols into p_symbols for plugin access.
 *****************************************************************************/
#define STORE_SYMBOLS( p_symbols ) \
    (p_symbols)->aout_OutputNextBuffer_inner = aout_OutputNextBuffer; \
    (p_symbols)->aout_FormatNbChannels_inner = aout_FormatNbChannels; \
    (p_symbols)->aout_FormatPrepare_inner = aout_FormatPrepare; \
    (p_symbols)->aout_FormatPrint_inner = aout_FormatPrint; \
    (p_symbols)->aout_FormatsPrint_inner = aout_FormatsPrint; \
    (p_symbols)->aout_FormatPrintChannels_inner = aout_FormatPrintChannels; \
    (p_symbols)->aout_FifoPop_inner = aout_FifoPop; \
    (p_symbols)->aout_FifoFirstDate_inner = aout_FifoFirstDate; \
    (p_symbols)->aout_VolumeSoftInit_inner = aout_VolumeSoftInit; \
    (p_symbols)->aout_VolumeNoneInit_inner = aout_VolumeNoneInit; \
    (p_symbols)->__aout_New_inner = __aout_New; \
    (p_symbols)->aout_Delete_inner = aout_Delete; \
    (p_symbols)->aout_DateInit_inner = aout_DateInit; \
    (p_symbols)->aout_DateSet_inner = aout_DateSet; \
    (p_symbols)->aout_DateMove_inner = aout_DateMove; \
    (p_symbols)->aout_DateGet_inner = aout_DateGet; \
    (p_symbols)->aout_DateIncrement_inner = aout_DateIncrement; \
    (p_symbols)->__aout_DecNew_inner = __aout_DecNew; \
    (p_symbols)->aout_DecDelete_inner = aout_DecDelete; \
    (p_symbols)->aout_DecNewBuffer_inner = aout_DecNewBuffer; \
    (p_symbols)->aout_DecDeleteBuffer_inner = aout_DecDeleteBuffer; \
    (p_symbols)->aout_DecPlay_inner = aout_DecPlay; \
    (p_symbols)->__aout_VolumeGet_inner = __aout_VolumeGet; \
    (p_symbols)->__aout_VolumeSet_inner = __aout_VolumeSet; \
    (p_symbols)->__aout_VolumeInfos_inner = __aout_VolumeInfos; \
    (p_symbols)->__aout_VolumeUp_inner = __aout_VolumeUp; \
    (p_symbols)->__aout_VolumeDown_inner = __aout_VolumeDown; \
    (p_symbols)->__aout_VolumeMute_inner = __aout_VolumeMute; \
    (p_symbols)->aout_Restart_inner = aout_Restart; \
    (p_symbols)->aout_FindAndRestart_inner = aout_FindAndRestart; \
    (p_symbols)->aout_ChannelsRestart_inner = aout_ChannelsRestart; \
    (p_symbols)->__config_GetType_inner = __config_GetType; \
    (p_symbols)->__config_GetInt_inner = __config_GetInt; \
    (p_symbols)->__config_PutInt_inner = __config_PutInt; \
    (p_symbols)->__config_GetFloat_inner = __config_GetFloat; \
    (p_symbols)->__config_PutFloat_inner = __config_PutFloat; \
    (p_symbols)->__config_GetPsz_inner = __config_GetPsz; \
    (p_symbols)->__config_PutPsz_inner = __config_PutPsz; \
    (p_symbols)->__config_LoadCmdLine_inner = __config_LoadCmdLine; \
    (p_symbols)->config_GetHomeDir_inner = config_GetHomeDir; \
    (p_symbols)->__config_LoadConfigFile_inner = __config_LoadConfigFile; \
    (p_symbols)->__config_SaveConfigFile_inner = __config_SaveConfigFile; \
    (p_symbols)->__config_ResetAll_inner = __config_ResetAll; \
    (p_symbols)->config_FindConfig_inner = config_FindConfig; \
    (p_symbols)->config_Duplicate_inner = config_Duplicate; \
    (p_symbols)->config_SetCallbacks_inner = config_SetCallbacks; \
    (p_symbols)->config_UnsetCallbacks_inner = config_UnsetCallbacks; \
    (p_symbols)->input_ExtractPES_inner = input_ExtractPES; \
    (p_symbols)->input_DeletePES_inner = input_DeletePES; \
    (p_symbols)->InitBitstream_inner = InitBitstream; \
    (p_symbols)->NextDataPacket_inner = NextDataPacket; \
    (p_symbols)->BitstreamNextDataPacket_inner = BitstreamNextDataPacket; \
    (p_symbols)->UnalignedShowBits_inner = UnalignedShowBits; \
    (p_symbols)->UnalignedRemoveBits_inner = UnalignedRemoveBits; \
    (p_symbols)->UnalignedGetBits_inner = UnalignedGetBits; \
    (p_symbols)->CloseBitstream_inner = CloseBitstream; \
    (p_symbols)->CurrentPTS_inner = CurrentPTS; \
    (p_symbols)->NextPTS_inner = NextPTS; \
    (p_symbols)->DecoderError_inner = DecoderError; \
    (p_symbols)->__input_SetStatus_inner = __input_SetStatus; \
    (p_symbols)->__input_SetRate_inner = __input_SetRate; \
    (p_symbols)->__input_Seek_inner = __input_Seek; \
    (p_symbols)->__input_Tell_inner = __input_Tell; \
    (p_symbols)->input_DumpStream_inner = input_DumpStream; \
    (p_symbols)->input_OffsetToTime_inner = input_OffsetToTime; \
    (p_symbols)->input_ToggleES_inner = input_ToggleES; \
    (p_symbols)->input_ChangeArea_inner = input_ChangeArea; \
    (p_symbols)->input_ChangeProgram_inner = input_ChangeProgram; \
    (p_symbols)->input_InitStream_inner = input_InitStream; \
    (p_symbols)->input_EndStream_inner = input_EndStream; \
    (p_symbols)->input_FindProgram_inner = input_FindProgram; \
    (p_symbols)->input_AddProgram_inner = input_AddProgram; \
    (p_symbols)->input_DelProgram_inner = input_DelProgram; \
    (p_symbols)->input_SetProgram_inner = input_SetProgram; \
    (p_symbols)->input_AddArea_inner = input_AddArea; \
    (p_symbols)->input_DelArea_inner = input_DelArea; \
    (p_symbols)->input_FindES_inner = input_FindES; \
    (p_symbols)->input_AddES_inner = input_AddES; \
    (p_symbols)->input_DelES_inner = input_DelES; \
    (p_symbols)->input_SelectES_inner = input_SelectES; \
    (p_symbols)->input_UnselectES_inner = input_UnselectES; \
    (p_symbols)->input_DecodePES_inner = input_DecodePES; \
    (p_symbols)->input_NullPacket_inner = input_NullPacket; \
    (p_symbols)->input_ClockManageControl_inner = input_ClockManageControl; \
    (p_symbols)->input_ClockManageRef_inner = input_ClockManageRef; \
    (p_symbols)->input_ClockGetTS_inner = input_ClockGetTS; \
    (p_symbols)->input_InfoCategory_inner = input_InfoCategory; \
    (p_symbols)->input_AddInfo_inner = input_AddInfo; \
    (p_symbols)->input_BuffersEnd_inner = input_BuffersEnd; \
    (p_symbols)->input_NewBuffer_inner = input_NewBuffer; \
    (p_symbols)->input_ReleaseBuffer_inner = input_ReleaseBuffer; \
    (p_symbols)->input_ShareBuffer_inner = input_ShareBuffer; \
    (p_symbols)->input_NewPacket_inner = input_NewPacket; \
    (p_symbols)->input_DeletePacket_inner = input_DeletePacket; \
    (p_symbols)->input_NewPES_inner = input_NewPES; \
    (p_symbols)->input_FillBuffer_inner = input_FillBuffer; \
    (p_symbols)->input_Peek_inner = input_Peek; \
    (p_symbols)->input_SplitBuffer_inner = input_SplitBuffer; \
    (p_symbols)->input_AccessInit_inner = input_AccessInit; \
    (p_symbols)->input_AccessReinit_inner = input_AccessReinit; \
    (p_symbols)->input_AccessEnd_inner = input_AccessEnd; \
    (p_symbols)->__intf_Eject_inner = __intf_Eject; \
    (p_symbols)->GetLang_1_inner = GetLang_1; \
    (p_symbols)->GetLang_2T_inner = GetLang_2T; \
    (p_symbols)->GetLang_2B_inner = GetLang_2B; \
    (p_symbols)->DecodeLanguage_inner = DecodeLanguage; \
    (p_symbols)->__module_Need_inner = __module_Need; \
    (p_symbols)->__module_Unneed_inner = __module_Unneed; \
    (p_symbols)->mstrtime_inner = mstrtime; \
    (p_symbols)->mdate_inner = mdate; \
    (p_symbols)->mwait_inner = mwait; \
    (p_symbols)->msleep_inner = msleep; \
    (p_symbols)->__sout_NewInstance_inner = __sout_NewInstance; \
    (p_symbols)->sout_DeleteInstance_inner = sout_DeleteInstance; \
    (p_symbols)->sout_FifoCreate_inner = sout_FifoCreate; \
    (p_symbols)->sout_FifoDestroy_inner = sout_FifoDestroy; \
    (p_symbols)->sout_FifoFree_inner = sout_FifoFree; \
    (p_symbols)->sout_FifoPut_inner = sout_FifoPut; \
    (p_symbols)->sout_FifoGet_inner = sout_FifoGet; \
    (p_symbols)->sout_FifoShow_inner = sout_FifoShow; \
    (p_symbols)->__sout_InputNew_inner = __sout_InputNew; \
    (p_symbols)->sout_InputDelete_inner = sout_InputDelete; \
    (p_symbols)->sout_InputSendBuffer_inner = sout_InputSendBuffer; \
    (p_symbols)->sout_BufferNew_inner = sout_BufferNew; \
    (p_symbols)->sout_BufferRealloc_inner = sout_BufferRealloc; \
    (p_symbols)->sout_BufferReallocFromPreHeader_inner = sout_BufferReallocFromPreHeader; \
    (p_symbols)->sout_BufferDelete_inner = sout_BufferDelete; \
    (p_symbols)->sout_BufferDuplicate_inner = sout_BufferDuplicate; \
    (p_symbols)->sout_BufferChain_inner = sout_BufferChain; \
    (p_symbols)->sout_AccessOutNew_inner = sout_AccessOutNew; \
    (p_symbols)->sout_AccessOutDelete_inner = sout_AccessOutDelete; \
    (p_symbols)->sout_AccessOutSeek_inner = sout_AccessOutSeek; \
    (p_symbols)->sout_AccessOutWrite_inner = sout_AccessOutWrite; \
    (p_symbols)->sout_MuxNew_inner = sout_MuxNew; \
    (p_symbols)->sout_MuxAddStream_inner = sout_MuxAddStream; \
    (p_symbols)->sout_MuxDeleteStream_inner = sout_MuxDeleteStream; \
    (p_symbols)->sout_MuxDelete_inner = sout_MuxDelete; \
    (p_symbols)->sout_MuxSendBuffer_inner = sout_MuxSendBuffer; \
    (p_symbols)->sout_cfg_parser_inner = sout_cfg_parser; \
    (p_symbols)->sout_stream_new_inner = sout_stream_new; \
    (p_symbols)->sout_stream_delete_inner = sout_stream_delete; \
    (p_symbols)->sout_SAPNew_inner = sout_SAPNew; \
    (p_symbols)->sout_SAPSend_inner = sout_SAPSend; \
    (p_symbols)->sout_SAPDelete_inner = sout_SAPDelete; \
    (p_symbols)->__var_Create_inner = __var_Create; \
    (p_symbols)->__var_Destroy_inner = __var_Destroy; \
    (p_symbols)->__var_Change_inner = __var_Change; \
    (p_symbols)->__var_Type_inner = __var_Type; \
    (p_symbols)->__var_Set_inner = __var_Set; \
    (p_symbols)->__var_Get_inner = __var_Get; \
    (p_symbols)->__var_AddCallback_inner = __var_AddCallback; \
    (p_symbols)->__var_DelCallback_inner = __var_DelCallback; \
    (p_symbols)->__vout_Request_inner = __vout_Request; \
    (p_symbols)->__vout_Create_inner = __vout_Create; \
    (p_symbols)->vout_Destroy_inner = vout_Destroy; \
    (p_symbols)->vout_VarCallback_inner = vout_VarCallback; \
    (p_symbols)->vout_ChromaCmp_inner = vout_ChromaCmp; \
    (p_symbols)->vout_CreatePicture_inner = vout_CreatePicture; \
    (p_symbols)->vout_InitPicture_inner = vout_InitPicture; \
    (p_symbols)->vout_AllocatePicture_inner = vout_AllocatePicture; \
    (p_symbols)->vout_DestroyPicture_inner = vout_DestroyPicture; \
    (p_symbols)->vout_DisplayPicture_inner = vout_DisplayPicture; \
    (p_symbols)->vout_DatePicture_inner = vout_DatePicture; \
    (p_symbols)->vout_LinkPicture_inner = vout_LinkPicture; \
    (p_symbols)->vout_UnlinkPicture_inner = vout_UnlinkPicture; \
    (p_symbols)->vout_PlacePicture_inner = vout_PlacePicture; \
    (p_symbols)->vout_CreateSubPicture_inner = vout_CreateSubPicture; \
    (p_symbols)->vout_DestroySubPicture_inner = vout_DestroySubPicture; \
    (p_symbols)->vout_DisplaySubPicture_inner = vout_DisplaySubPicture; \
        (p_symbols)->vlc_strdup_inner = vlc_strdup; \
            (p_symbols)->vlc_strndup_inner = vlc_strndup; \
        (p_symbols)->vlc_atof_inner = vlc_atof; \
        (p_symbols)->vlc_getenv_inner = vlc_getenv; \
            (p_symbols)->vlc_strcasecmp_inner = vlc_strcasecmp; \
            (p_symbols)->vlc_strncasecmp_inner = vlc_strncasecmp; \
        (p_symbols)->vlc_lseek_inner = vlc_lseek; \
    (p_symbols)->vlc_dgettext_inner = vlc_dgettext; \
    (p_symbols)->vlc_error_inner = vlc_error; \
    (p_symbols)->__intf_Create_inner = __intf_Create; \
    (p_symbols)->intf_RunThread_inner = intf_RunThread; \
    (p_symbols)->intf_StopThread_inner = intf_StopThread; \
    (p_symbols)->intf_Destroy_inner = intf_Destroy; \
    (p_symbols)->__msg_Generic_inner = __msg_Generic; \
    (p_symbols)->__msg_Info_inner = __msg_Info; \
    (p_symbols)->__msg_Err_inner = __msg_Err; \
    (p_symbols)->__msg_Warn_inner = __msg_Warn; \
    (p_symbols)->__msg_Dbg_inner = __msg_Dbg; \
    (p_symbols)->__msg_Subscribe_inner = __msg_Subscribe; \
    (p_symbols)->__msg_Unsubscribe_inner = __msg_Unsubscribe; \
    (p_symbols)->__vlc_object_create_inner = __vlc_object_create; \
    (p_symbols)->__vlc_object_destroy_inner = __vlc_object_destroy; \
    (p_symbols)->__vlc_object_attach_inner = __vlc_object_attach; \
    (p_symbols)->__vlc_object_detach_inner = __vlc_object_detach; \
    (p_symbols)->__vlc_object_get_inner = __vlc_object_get; \
    (p_symbols)->__vlc_object_find_inner = __vlc_object_find; \
    (p_symbols)->__vlc_object_yield_inner = __vlc_object_yield; \
    (p_symbols)->__vlc_object_release_inner = __vlc_object_release; \
    (p_symbols)->__vlc_list_find_inner = __vlc_list_find; \
    (p_symbols)->vlc_list_release_inner = vlc_list_release; \
    (p_symbols)->playlist_Command_inner = playlist_Command; \
    (p_symbols)->playlist_Add_inner = playlist_Add; \
    (p_symbols)->playlist_AddName_inner = playlist_AddName; \
    (p_symbols)->playlist_AddItem_inner = playlist_AddItem; \
    (p_symbols)->playlist_Delete_inner = playlist_Delete; \
    (p_symbols)->playlist_Move_inner = playlist_Move; \
    (p_symbols)->playlist_LoadFile_inner = playlist_LoadFile; \
    (p_symbols)->playlist_SaveFile_inner = playlist_SaveFile; \
    (p_symbols)->__vlc_threads_init_inner = __vlc_threads_init; \
    (p_symbols)->__vlc_threads_end_inner = __vlc_threads_end; \
    (p_symbols)->__vlc_mutex_init_inner = __vlc_mutex_init; \
    (p_symbols)->__vlc_mutex_destroy_inner = __vlc_mutex_destroy; \
    (p_symbols)->__vlc_cond_init_inner = __vlc_cond_init; \
    (p_symbols)->__vlc_cond_destroy_inner = __vlc_cond_destroy; \
    (p_symbols)->__vlc_thread_create_inner = __vlc_thread_create; \
    (p_symbols)->__vlc_thread_set_priority_inner = __vlc_thread_set_priority; \
    (p_symbols)->__vlc_thread_ready_inner = __vlc_thread_ready; \
    (p_symbols)->__vlc_thread_join_inner = __vlc_thread_join; \
    (p_symbols)->__vout_SynchroInit_inner = __vout_SynchroInit; \
    (p_symbols)->vout_SynchroRelease_inner = vout_SynchroRelease; \
    (p_symbols)->vout_SynchroReset_inner = vout_SynchroReset; \
    (p_symbols)->vout_SynchroChoose_inner = vout_SynchroChoose; \
    (p_symbols)->vout_SynchroTrash_inner = vout_SynchroTrash; \
    (p_symbols)->vout_SynchroDecode_inner = vout_SynchroDecode; \
    (p_symbols)->vout_SynchroEnd_inner = vout_SynchroEnd; \
    (p_symbols)->vout_SynchroDate_inner = vout_SynchroDate; \
    (p_symbols)->vout_SynchroNewPicture_inner = vout_SynchroNewPicture; \
    (p_symbols)->vout_ShowTextRelative_inner = vout_ShowTextRelative; \
    (p_symbols)->vout_ShowTextAbsolute_inner = vout_ShowTextAbsolute; \

